load 'solve_sos_equation.sage'

# Examples from Reznick, Some concrete aspects of Hilbert's 17th problem
# Motzkin M
if 1==0:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^4 * y^2 + x^2 * y^4 + z^6 - 3 * x^2 * y^2 * z^2')
  den_degree = 2

# Robinson R
if 1==0:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^6 + y^6 + z^6 - (x^4 * y^2 + x^2 * y^4 + x^4 * z^2 + x^2 * z^4 + y^4 * z^2 + y^2 * z^4) + 3 * x^2 * y^2 * z^2')
  den_degree = 1

# Robinson f
if 1==0:
  indeterminates=['x','y','z','w']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^2 * (x-w)^2 + y^2 * (y-w)^2 + z^2 * (z-w)^2 + 2*x*y*z * (x+y+z-2*w)')
  den_degree = 1

# Choi-Lam F works with scaling factor 10^15
if 1==0:
  indeterminates=['x1','x2','x3','y1','y2','y3']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x1^2 * y1^2 + x2^2 * y2^2 + x3^2 * y3^2 + 2*(x1^2 * y2^2 + x2^2 * y3^2 + x3^2 * y1^2) - 2*x1*x2*y1*y2 - 2*x1*x3*y1*y3 - 2*x2*x3*y2*y3')
  den_degree=1

# Choi-Lam Q
if 1==0:
  indeterminates=['x', 'y', 'z', 'w']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^2*y^2 + x^2*z^2 + y^2*z^2 + w^4 -4*w*x*y*z')
  den_degree=1

# Choi-Lam S
if 1==0:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^4 * y^2 + y^4 * z^2 + z^4 * x^2 - 3*x^2*y^2*z^2')
  den_degree=1

# Choi-Lam H
if 1==1:
  indeterminates=['x1', 'x2', 'x3', 'x4', 'x5']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('(x1^2 + x2^2 + x3^2 + x4^2 + x5^2)^2 - 4* (x1^2 * x2^2 + x2^2 * x3^2 + x3^2 * x4^2 + x4^2 * x5^2 + x5^2 * x1^2)');
  den_degree = 1
    
# Schmuedgen q
if 1==0:
  indeterminates=['x','y','z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('200*(x^3-4*x*z^2)^2 + 200*(y^3-4*y*z^2)^2 + (y^2-x^2)*x*(x+2*z)*(x^2-2*x*z+2*y^2-8*z^2)')
  den_degree=1
  
def homogeneous_monomials(indets, degree):
  if len(indets)==0 or degree==0:
    return [polynomial_ring(1)]
  if len(indets)==1:
    return [polynomial_ring(indets[0])^degree]
  return [polynomial_ring(indets[0])^d * m
          for d in xrange(degree+1)
          for m in homogeneous_monomials(indets[1:], degree-d)]

def all_monomials(indets, degree):
  if len(indets)==0 or degree==0:
    return [polynomial_ring(1)]
  if len(indets)==1:
    return [polynomial_ring(indets[0])^k for k in xrange(degree+1)]
  return [polynomial_ring(indets[0])^d * m
          for d in xrange(degree+1)
          for m in all_monomials(indets[1:], degree-d)]
  
assert(polynomial.degree() % 2 == 0)
  
lhs_polynomials=[polynomial, -1]

assert polynomial.is_homogeneous()
if polynomial.is_homogeneous():
  lhs_monomial_lists=[homogeneous_monomials(indeterminates, den_degree), homogeneous_monomials(indeterminates, den_degree+polynomial.degree()/2)]
else:
  lhs_monomial_lists=[all_monomials(indeterminates, den_degree), all_monomials(indeterminates, den_degree+polynomial.degree()/2)]

print "Numbers of monomials:", [len(l) for l in lhs_monomial_lists]

rhs_polynomial=polynomial_ring(0)

solution = solve_sos_equation(polynomial_ring, lhs_polynomials, lhs_monomial_lists, rhs_polynomial)

if solution == ():
  print 'NO SOLUTION'
else:
  den = apply_sos(solution[0], lhs_monomial_lists[0])
  num = apply_sos(solution[1], lhs_monomial_lists[1])
  if den == 0:
    print 'ZERO SOLUTION'
  else:
    assert(polynomial == num/den)
    print 'SOLUTION'
    print 'NUM'
    print num
    print 'DEN'
    print den
