# This file was *autogenerated* from the file quadratic_forms.sage.
from sage.all_cmdline import *   # import sage library
_sage_const_3 = Integer(3); _sage_const_2 = Integer(2); _sage_const_1 = Integer(1); _sage_const_0 = Integer(0)
def is_positive_definite_sylvester(m):
    assert m.is_symmetric()
    for i in xrange(_sage_const_1 ,m.nrows()+_sage_const_1 ):
        if m[_sage_const_0 :i,_sage_const_0 :i].determinant() <= _sage_const_0 :
            return False
    return True

def is_positive_semidefinite_sylvester(m):
    if m.is_zero():
        return True
    b = Matrix(VectorSpace(QQ,_sage_const_3 ).subspace(m.rows()).basis())
    return is_positive_definite_sylvester(b * m * b.transpose())

class IsNotPositiveSemidefinite(ValueError):
    def __init__ (self, matrix):
        self.matrix = matrix
    def __str__(self):
        return 'Is not positive semidefinite:\n' + self.matrix.str()

class IsNotSymmetric(ValueError):
    def __init__ (self, matrix):
        self.matrix = matrix
    def __str__(self):
        return 'Is not symmetric:\n' + self.matrix.str()

def tvv_matrix(v):
    return transpose(v) * Matrix([v])

def sdp_matrix_to_sos(m0):
    ret = []
    m=copy(m0)
    if not m.is_symmetric():
        raise IsNotSymmetric(m)
    dim = m.nrows()
    for i in xrange(dim):
        coeff = m[i, i]
        if coeff < _sage_const_0 :
            raise IsNotPositiveSemidefinite(m0)
        if coeff == _sage_const_0 :
            if not m.row(i).is_zero():
                raise IsNotPositiveSemidefinite(m0)
        else:
            v = m.row(i) / coeff
            m -= tvv_matrix(v) * coeff
            ret.append((coeff, v))
    return ret

def sos_to_sdp_matrix(sos):
    if len(sos)==_sage_const_0 :
        return _sage_const_0
    (coeff, v) = sos[_sage_const_0 ]
    dim = len(v)
    ret = Matrix(dim)
    for (coeff, v) in sos:
        ret += coeff * tvv_matrix(v)
    return ret

def is_positive_semidefinite(m):
    try:
        sdp_matrix_to_sos(m)
        return True
    except IsNotPositiveSemidefinite:
        return False

def apply_sos(sos, monomials):
    r= _sage_const_0
    for (coeff, v) in sos:
        r += coeff * (v * vector(monomials))**_sage_const_2
    return r

def apply_qmatrix(q, v):
  return v*q*v

def matrix_to_gauss(m0):
  ret = []
  m=copy(m0)
  if not m.is_symmetric():
    raise IsNotSymmetric(m)
  dim = m.nrows()
  for i in xrange(dim):
    if m.row(i) == _sage_const_0 :
      continue
    for j in xrange(i, dim):
      coeff = m[j, j]
      if coeff != _sage_const_0 :
        break
    if coeff != _sage_const_0 :
      v = m.row(j) / coeff
      m -= tvv_matrix(v) * coeff
      ret.append((coeff, v))
    else:
      # Only zeroes on the diagonal
      rowi = m.row(i)
      for j in xrange(i, dim):
        coeff = rowi[j]
        if coeff != _sage_const_0 :
          break
      assert coeff != _sage_const_0  # otherwise we'd have skipped the row
      rowi[i] /= coeff
      rowj = m.row(j) /coeff
      rowi[i] = _sage_const_1
      rowi[j] = _sage_const_0
      rowj[i] = _sage_const_0
      rowj[j] = _sage_const_1
      u = rowi + rowj
      v = rowi - rowj
      coeff /= _sage_const_2
      m -= tvv_matrix(u) * coeff
      m += tvv_matrix(v) * coeff
      ret.append((coeff, u))
      ret.append((-coeff, v))
  return ret

def signature(q):
  signs = [sgn(x) for (x,v) in matrix_to_gauss(q)]
  return (signs.count(_sage_const_1 ), signs.count(_sage_const_0 ), signs.count(-_sage_const_1 ))

def signature_from_evs(q):
  signs = map(sgn, q.eigenvalues())
  return (count(signs, _sage_const_1 ), count(signs, _sage_const_0 ), count(signs, -_sage_const_1 ))
